@@ -1,6 +1,8 @@
 #!/usr/bin/perl
-# $Source: /usr/src/CVS/mnogosearch/install.pl,v $
-# $Id: install.pl,v 1.5 2001/08/21 11:31:57 gluke Exp $
+# $Source: /usr/home/bar/cvsroot/mnogosearch32/install.pl,v $
+# $Id: install.pl,v 1.4 2003/06/19 20:51:34 maxime Exp $
+
+use Cwd 'abs_path';
 
 $prefix="/usr/local/mnogosearch";
 
@@ -15,22 +17,12 @@ $build_static='yes';
 $syslog='yes';
 $syslog_facility='default';
 
-$linux_threads='no';
-$bsd_threads='no';
-
-$guesser='no';
+$pthreads='yes';
 $parser='yes';
 $mp3='yes';
-$aliasprog='yes';
-$ext_news='no';
+$dmalloc='no';
+$zlib='yes';
 
-$cache='yes';
-$fast_tag='no';
-$fast_cat='no';
-$fast_site='no';
-$phrase='no';
-
-$cgi='yes';
 $file='yes';
 $http='yes';
 $ftp='yes';
@@ -43,16 +35,14 @@ $conf_file='install.options';
 
 $|=1;
 
-print "mnoGoSeach installation script version 0.1\n\n";
+print "mnoGoSeach installation script version 0.4\n\n";
 
 set_layout();
 set_db(); print "\n";
 set_libs(); print "\n";
 set_syslog(); print "\n";
-set_threads(); print "\n";
-set_other(); print "\n";
-set_cache(); print "\n";
 set_shema(); print "\n";
+set_other(); print "\n";
 
 configure();
 
@@ -99,26 +89,20 @@ sub configure {
         	} else {
         		push @arg,"--with-openssl";
         	}
-        } 
+        }
 
-        if ($linux_threads eq 'yes') {
-        	push @arg,"--enable-linux-pthreads";
-        } elsif ($linux_threads eq 'no') {
-        	push @arg,"--disable-linux-pthreads";
+        if ($pthreads eq 'yes') {
+        	push @arg,"--enable-pthreads";
+        } elsif ($pthreads eq 'no') {
+        	push @arg,"--disable-pthreads";
 	}
 
-        if ($bsd_threads eq 'yes') {
-        	push @arg,"--enable-freebsd-pthreads";
-        } elsif ($bsd_threads eq 'no') {
-        	push @arg,"--disable-freebsd-pthreads";
+        if ($dmalloc eq 'yes') {
+        	push @arg,"--enable-dmalloc";
+        } elsif ($pthreads eq 'no') {
+        	push @arg,"--disable-dmalloc";
 	}
 
-        if ($guesser eq 'yes') {
-        	push @arg,"--enable-charset-guesser";
-        } elsif ($guesser eq 'no') {
-        	push @arg,"--disable-charset-guesser";
-	}
-        
         if ($parser eq 'yes') {
         	push @arg,"--enable-parser";
         } elsif ($parser eq 'no') {
@@ -131,47 +115,9 @@ sub configure {
         	push @arg,"--disable-mp3";
 	}
 
-        if ($aliasprog eq 'yes') {
-        	push @arg,"--enable-aliasprog";
-        } elsif ($aliasprog eq 'no') {
-        	push @arg,"--disable-aliasprog";
-	}
-
-        if ($ext_news eq 'yes') {
-        	push @arg,"--enable-news-extension";
-        } elsif ($ext_news eq 'no') {
-        	push @arg,"--disable-news-extension";
-	}
-
-        if ($fast_tag eq 'yes') {
-        	push @arg,"--enable-fast-tag";
-        } elsif ($fast_tag eq 'no') {
-        	push @arg,"--disable-fast-tag";
-	}
-
-        if ($fast_cat eq 'yes') {
-        	push @arg,"--enable-fast-cat";
-        } elsif ($fast_cat eq 'no') {
-        	push @arg,"--disable-fast-cat";
-	}
-
-        if ($fast_site eq 'yes') {
-        	push @arg,"--enable-fast-site";
-        } elsif ($fast_site eq 'no') {
-        	push @arg,"--disable-fast-site";
-	}
-
-        if ($phrase eq 'yes') {
-        	push @arg,"--enable-phrase";
-        } elsif ($phrase eq 'no') {
-        	push @arg,"--disable-phrase";
-	}
-
-        if ($cgi eq 'yes') {
-        	push @arg,"--enable-exec-cgi";
-        } elsif ($cgi eq 'no') {
-        	push @arg,"--disable-exec-cgi";
-	}
+        if ($zlib eq 'no') {
+        	push @arg,"--without-zlib";
+        }
 
         if ($file eq 'yes') {
         	push @arg,"--enable-file";
@@ -203,7 +149,11 @@ sub configure {
         	push @arg,"--disable-news";
 	}
 
-        push @arg,"$db_string";
+	$db_string=~s/^\s+//;
+	$db_string=~s/\s+$//;
+	@db_string=split(/\s+/,$db_string);
+	
+        push @arg,@db_string;
 
         print "@arg\n\n";
 	
@@ -221,11 +171,13 @@ sub configure {
 	print "Now you should run 'make' then 'make install' to finish installation.\n";
 	print "To configure package with the same options you can run ./$conf_file\n\n";
 }
- 
+
 # ---------------------------
 # set_db
 # ---------------------------
 sub set_db {
+	print "Database settings\n";
+	print "-----------------\n";
 	while () {
 		print "Include built-in database support? (Use for small sites) (yes/no) [$builtin]: ";
 		$temp=read_in();
@@ -240,6 +192,8 @@ sub set_db {
 	if ($builtin eq 'yes') {
 		$db_string='--with-built-in';
 		return;
+	} else {
+		$db_string='';
 	}
 	
 	while () {
@@ -251,7 +205,7 @@ sub set_db {
 			$auto=$temp;
 			last;
 		}
-	}	
+	}
 	
 	if ($auto eq 'yes') {
 		print "\n";
@@ -259,92 +213,140 @@ sub set_db {
 		    (-f "/usr/include/mysql/mysql.h") ||
 		    (-f "/usr/local/include/mysql/mysql.h") ||
 		    (-f "/usr/include/mysql.h") ||
-		    (-f "/usr/local/include/mysql.h")) {		    
-		    	$db=1;
+		    (-f "/usr/local/include/mysql.h")) {
+		    	push @db_found2,1;
+			$db{mysql}=1;
 			print "MySQL server found\n";
-		} elsif ((-f "/usr/include/pgsql/libpq-fe.h") ||
+		}
+		
+		if ((-f "/usr/include/pgsql/libpq-fe.h") ||
 		         (-f "/usr/lib/libpq.a") ||
 			 (-f "/usr/local/pgsql/include/libpq-fe.h") ||
 		         (-f "/usr/local/pgsql/lib/libpq.a") ||
 			 (-f "/usr/local/include/libpq-fe.h") ||
 		         (-f "/usr/local/lib/libpq.a")) {
-			$db=2;
+			push @db_found2,2;
+			$db{pgsql}=1;
 			print "PostgreSQL server found\n";
-		} elsif ((-f "/usr/local/Hughes/include/msql.h") ||		
+		}
+		
+		if ((-f "/usr/local/Hughes/include/msql.h") ||		
 		         (-f "/usr/Hughes/include/msql.h") ||
 			 (-f "/usr/local/include/msql.h") ||		
 			 (-f "/usr/include/msql.h")) {
-			$db=3;
+			push @db_found2,3;
+			$db{msql}=1;
 			print "mSQL server found\n";
-		} elsif ((-f "/usr/local/include/isql.h") ||
-			 (-f "/usr/include/isql.h")) {
-			$db=10; 		         
-			print "iODBC stuff found\n";
-		} elsif ((-f "/usr/local/include/sql.h") ||
-			 (-f "/usr/include/sql.h")) {			 
-			$db=11;		         			 			
-			print "unixODBC stuff found\n";
-		} elsif ((-f "/usr/local/include/cli0cli.h") ||
+		}
+		
+		if (-f "/opt/sybase/include/ctpublic.h") {
+			push @db_found2,4;
+			$db{ctlib}=1;
+			print "Ct-Lib stuff found\n";
+		}
+		
+		if (-f "/usr/local/include/ctpublic.h") {
+			push @db_found2,5;
+			$db{freetds}=1;
+			print "FreeTDS Ct-Lib stuff found\n";
+		}
+		
+		if ((-f "/usr/interbase/include/ibase.h") ||
+			 (-f "/usr/local/interbase/include/ibase.h")) {
+			push @db_found2,6;
+			$db{ibase}=1;
+			print "Interbase server found\n";
+		}
+		
+		if (-f "$ENV{ORA_HOME}/rdbms/demo/ocidfn.h") {
+			push @db_found2,7;
+			$db{ora7}=1;
+			print "Oracle 7.x server found\n";
+		}
+		
+		if (-f "/oracle8/app/oracle/product/8.0.5/rdbms/demo/oci.h") {
+			push @db_found2,8;
+			$db{ora8}=1;
+			print "Oracle 8.0.x server found\n";
+		}
+		
+		if (-f "$ENV{ORA_HOME}/rdbms/demo/oci.h") {
+			push @db_found2,9;
+			$db{ora8i}=1;
+			print "Oracle 8.x server found\n";
+		}
+		
+		if ((-f "/usr/local/include/cli0cli.h") ||
 		         (-f "/usr/include/cli0cli.h")) {
-			$db=4; 
-			print "Solid server found\n";
-		} elsif ((-f "/usr/local/virtuoso-ent/odbcsdk/include/isql.h") ||
+			push @db_found3,1; 
+			print "Solid server found\n";			
+		}
+		
+		if ((-f "/usr/sapdb/incl/sql.h") ||
+			 (-f "/usr/local/sapdb/incl/sql.h")) {
+			push @db_found3,2;
+			print "SAPDB server found\n";			
+		}
+		
+		if ((-f "/usr/local/include/isql.h") ||
+			 (-f "/usr/include/isql.h")) {
+			push @db_found,1;
+			print "iODBC stuff found\n";
+		}
+		
+		if ((-f "/usr/local/include/sql.h") ||
+			 (-f "/usr/include/sql.h")) {
+			push @db_found,2;
+			print "unixODBC stuff found\n";
+		}
+		
+		if ((-f "/usr/local/virtuoso-ent/odbcsdk/include/isql.h") ||
 			 (-f "/usr/virtuoso-ent/odbcsdk/include/isql.h") ||
 			 (-f "/usr/local/virtuoso-lite/odbcsdk/include/isql.h") ||
 			 (-f "/usr/virtuoso-lite/odbcsdk/include/isql.h") ||
 			 (-f "/usr/local/virtuoso/odbcsdk/include/isql.h") ||
 			 (-f "/usr/virtuoso/odbcsdk/include/isql.h")) {
-			$db=12;
+			push @db_found,3;
 			print "OpenLink stuff found\n";
-		} elsif ((-f "/usr/local/easysoft/oob/client/include/sql.h") ||
-		         (-f "/usr/easysoft/oob/client/include/sql.h")) {			 
-			$db=13;
+		}
+		
+		if ((-f "/usr/local/easysoft/oob/client/include/sql.h") ||
+		         (-f "/usr/easysoft/oob/client/include/sql.h")) {
+			push @db_found,4;
 			print "EasySoft stuff found\n";
-		} elsif ((-f "/usr/sapdb/incl/sql.h") ||
-			 (-f "/usr/local/sapdb/incl/sql.h")) {
-			$db=5;
-			print "SAPDB server found\n";
-		} elsif ((-f "/usr/interbase/include/ibase.h") ||
-			 (-f "/usr/local/interbase/include/ibase.h")) {
-			$db=6;
-			print "Interbase server found\n";
-		} elsif (-f "$ENV{ORA_HOME}/rdbms/demo/ocidfn.h") {
-			$db=7;
-			print "Oracle 7.x server found\n";
-		} elsif (-f "/oracle8/app/oracle/product/8.0.5/rdbms/demo/oci.h") {
-			$db=8;
-			print "Oracle 8.0.x server found\n";
-		} elsif (-f "$ENV{ORA_HOME}/rdbms/demo/oci.h") {
-			$db=9;
-			print "Oracle 8.x server found\n";
-		} else {
-			$db='';
-			print "Sorry, no SQL servers at known location found\n";
+		} 
+
+		if ((-f "/usr/local/include/sqlcli1.h") ||
+		    (-f "/home/db2/include/sqlcli1.h")) {			 
+			push @db_found,5;
+			print "IBM DB2 stuff found\n";
+		} 
+		
+		$db_found=@db_found;
+		$db_found2=@db_found2;
+		$db_found3=@db_found3;
+		if (($db_found == 0) && ($db_found2 == 0) && ($db_found3 == 0)) {
+			print "Sorry, no SQL servers at known locations was found\n";
 		}
 	}
 	
 	while () {
-		print "\nWhich database support include?\n";
+		print "\nWhich ODBC-style database support to include?\n";
+		print "Note, that you may choose only one from the following.\n";
 		
-		print "  1 - Include MySQL support.\n";
-		print "  2 - Include PostgreSQL support.\n";
-		print "  3 - Include mSQL support.\n";
-		print "  4 - Include Solid support.\n";		
-		print "  5 - Include SAPDB support.\n";
-		print "  6 - Include InterBase support.\n";
-		print "  7 - Include Oracle 7.3 support.\n";
-		print "  8 - Include Oracle 8 support.\n";
-		print "  9 - Include Oracle 8i support.\n";
-		print " 10 - Include iODBC support.\n";
-		print " 11 - Include unixODBC support.\n";
-		print " 12 - Include OpenLink ODBC support.\n";
-		print " 13 - Include EasySoft ODBC support.\n";		
+		print "  1 - Include iODBC support.\n";
+		print "  2 - Include unixODBC support.\n";
+		print "  3 - Include OpenLink ODBC support.\n";
+		print "  4 - Include EasySoft ODBC support.\n";		
+		print "  5 - Include IBM DB2 support.\n";		
+		print "  6 - None of these.\n";		
 		
-		if ($db eq '') {
-			print "\nChoose one from the mentioned: ";
+		if ($db_found == 0) {
+			$db='6';
 		} else {
-			print "\nChoose one from the mentioned ($db):";
+			$db=$db_found[0];
 		}
+		print "\nChoose one from the mentioned ($db): ";
 		
 		$temp=read_in();
 		if ($temp eq '') {
@@ -352,161 +354,265 @@ sub set_db {
 		}
 		
 		if ($temp eq '1') {
-			if ($temp eq $db) {
-				print "Enter MySQL base install directory [autodetect]: ";
-			} else {
-				print "Enter MySQL base install directory [/usr/local/mysql]: ";
-			}
-			
-			$temp=read_in();
-			if ($temp eq '') { $db_string="--with-mysql"; } 
-			else { $db_string="--with-mysql=$temp"; }
-			
-			last;
-		} elsif ($temp eq '2') {
-			if ($temp eq $db) {
-				print "Enter PostgreSQL base install directory [autodetect]: ";
-			} else {			
-				print "Enter PostgreSQL base install directory [/usr/local/pgsql]: ";
-			}
-			
-			$temp=read_in();
-			if ($temp eq '') { $db_string="--with-pgsql"; } 
-			else { $db_string="--with-pgsql=$temp"; }
-			
-			last;
-		} elsif ($temp eq '3') {
-			if ($temp eq $db) {
-				print "Enter mSQL base install directory [autodetect]: ";
-			} else {
-				print "Enter mSQL base install directory [/usr/local/Hughes]: ";
-			}
-			
-			$temp=read_in();
-			if ($temp eq '') { $db_string="--with-msql"; } 
-			else { $db_string="--with-msql=$temp"; }
-			
-			last;
-		} elsif ($temp eq '10') {
 			if ($temp eq $db) {
 				print "Enter iODBC base install directory [autodetect]: ";
 			} else {
 				print "Enter iODBC base install directory [/usr/local]: ";
 			}
 			
-			$temp=read_in();
-			if ($temp eq '') { $db_string="--with-iodbc"; } 
-			else { $db_string="--with-iodbc=$temp"; }
+			$temp=read_in_nolc();
+			if ($temp eq '') { $db_string .= " --with-iodbc "; } 
+			else { $db_string .= " --with-iodbc=$temp "; }
 			
 			last;
-		} elsif ($temp eq '11') {
+		} elsif ($temp eq '2') {
 			if ($temp eq $db) {
 				print "Enter unixODBC base install directory [autodetect]: ";
 			} else {
 				print "Enter unixODBC base install directory [/usr/local]: ";
 			}
 			
-			$temp=read_in();
-			if ($temp eq '') { $db_string="--with-unixODBC"; } 
-			else { $db_string="--with-unixODBC=$temp"; }
+			$temp=read_in_nolc();
+			if ($temp eq '') { $db_string .= " --with-unixODBC "; } 
+			else { $db_string .= " --with-unixODBC=$temp "; }
 			
 			last;
-		} elsif ($temp eq '4') {
-			if ($temp eq $db) {
-				print "Enter Solid base install directory [autodetect]: ";
-			} else {
-				print "Enter Solid base install directory [/usr/local]: ";
-			}
-			
-			$temp=read_in();
-			if ($temp eq '') { $db_string="--with-solid"; } 
-			else { $db_string="--with-solid=$temp"; }
-			
-			last;
-		} elsif ($temp eq '12') {
+		} elsif ($temp eq '3') {
 			if ($temp eq $db) {
 				print "Enter OpenLink ODBC base install directory [autodetect]: ";
 			} else {
 				print "Enter OpenLink ODBC base install directory [/usr/local/virtuoso/odbcsdk]: ";
 			}
 			
-			$temp=read_in();
-			if ($temp eq '') { $db_string="--with-openlink"; } 
-			else { $db_string="--with-openlink=$temp"; }
+			$temp=read_in_nolc();
+			if ($temp eq '') { $db_string .= " --with-openlink "; } 
+			else { $db_string .= " --with-openlink=$temp "; }
 			
 			last;
-		} elsif ($temp eq '13') {
+		} elsif ($temp eq '4') {
 			if ($temp eq $db) {
 				print "Enter EasySoft ODBC base install directory [autodetect]: ";
 			} else {
 				print "Enter EasySoft ODBC base install directory [/usr/local/easysoft/oob/client]: ";
 			}
 			
-			$temp=read_in();
-			if ($temp eq '') { $db_string="--with-easysoft"; } 
-			else { $db_string="--with-easysoft=$temp"; }
+			$temp=read_in_nolc();
+			if ($temp eq '') { $db_string .= " --with-easysoft "; } 
+			else { $db_string .= " --with-easysoft=$temp "; }
 			
 			last;
 		} elsif ($temp eq '5') {
+			if ($temp eq $db) {
+				print "Enter IBM DB2 base install directory [autodetect]: ";
+			} else {
+				print "Enter IBM DB2 base install directory [/home/db2]: ";
+			}
+			
+			$temp=read_in_nolc();
+			if ($temp eq '') { $db_string .= " --with-db2 "; } 
+			else { $db_string .= " --with-db2=$temp "; }
+			
+			last;
+		} elsif ($temp eq '6') {	
+		    last;
+		}
+	}
+
+
+	while () {
+		print "\nWhich database support to include?\n";
+		print "Note, that you can choose only one from these.\n";
+		
+		print "  1 - Include Solid support.\n";		
+		print "  2 - Include SAPDB support.\n";
+		print "  3 - None of these.\n";		
+		
+		if ($db_found3 == 0) {
+			$db='3';
+		} else {
+			$db=$db_found3[0];
+		}
+		
+		print "\nChoose one from the mentioned ($db):";
+		
+		$temp=read_in();
+		if ($temp eq '') {
+			$temp=$db;
+		}
+		
+		if ($temp eq '1') {
+			if ($temp eq $db) {
+				print "Enter Solid base install directory [autodetect]: ";
+			} else {
+				print "Enter Solid base install directory [/usr/local]: ";
+			}
+			
+			$temp=read_in_nolc();
+			if ($temp eq '') { $db_string .= " --with-solid "; } 
+			else { $db_string .= " --with-solid=$temp "; }
+			
+			last;
+		} elsif ($temp eq '2') {
 			if ($temp eq $db) {
 				print "Enter SAPDB base install directory [autodetect]: ";				
 			} else {
 				print "Enter SAPDB base install directory [/usr/sapdb]: ";
 			}
 			
-			$temp=read_in();
-			if ($temp eq '') { $db_string="--with-sapdb"; } 
-			else { $db_string="--with-sapdb=$temp"; }
+			$temp=read_in_nolc();
+			if ($temp eq '') { $db_string .= " --with-sapdb "; } 
+			else { $db_string .= " --with-sapdb=$temp "; }
 			
 			last;
-		} elsif ($temp eq 'ibase') {
-			if ($temp eq $db) {
-				print "Enter InterBase base install directory [autodetect]: ";
-			} else {
-				print "Enter InterBase base install directory [/usr/interbase]: ";
-			}
+		} elsif ($temp eq '3') {	
+		    last;
+		}
+	}
+
+	
+        if ($db{mysql}) { $db_auto='yes'; }
+	else {  $db_auto='no'; }
+	print "Include MySQL support [$db_auto] ? ";	
+	$temp=read_in();	
+	if (($temp eq 'yes') || (($temp eq '') && ($db_auto eq 'yes'))) {
+		if ($db{mysql}) {
+			print "Enter MySQL base install directory [autodetect]: ";
+		} else {
+			print "Enter MySQL base install directory [/usr/local/mysql]: ";
+		}
+		$temp=read_in_nolc();
+		if ($temp eq '') { $db_string .= " --with-mysql "; } 
+		else { $db_string .= " --with-mysql=$temp "; }
+	}
+	
+	if ($db{pgsql}) { $db_auto='yes'; }
+	else {  $db_auto='no'; }
+	print "Include PostgreSQL support [$db_auto] ? ";	
+	$temp=read_in();			
+	if (($temp eq 'yes') || (($temp eq '') && ($db_auto eq 'yes'))) {
+		if ($db{pgsql}) {
+			print "Enter PostgreSQL base install directory [autodetect]: ";
+		} else {			
+			print "Enter PostgreSQL base install directory [/usr/local/pgsql]: ";
+		}
 			
-			$temp=read_in();
-			if ($temp eq '') { $db_string="--with-ibase"; } 
-			else { $db_string="--with-ibase=$temp"; }
+		$temp=read_in_nolc();
+		if ($temp eq '') { $db_string .= " --with-pgsql "; } 
+		else { $db_string .= " --with-pgsql=$temp "; }
+	}			
+
+	if ($db{msql}) { $db_auto='yes'; }
+	else {  $db_auto='no'; }
+	print "Include mSQL support [$db_auto] ? ";	
+	$temp=read_in();			
+	if (($temp eq 'yes') || (($temp eq '') && ($db_auto eq 'yes'))) {
+		if ($db{msql}) {
+			print "Enter mSQL base install directory [autodetect]: ";
+		} else {
+			print "Enter mSQL base install directory [/usr/local/Hughes]: ";
+		}
 			
-			last;
-		} elsif ($temp eq '7') {
-			print "Enter Oracle home directory [\$ORA_HOME]: ";
+		$temp=read_in_nolc();
+		if ($temp eq '') { $db_string .= " --with-msql "; } 
+		else { $db_string .= " --with-msql=$temp "; }
+	}
+	
+	if ($db{ibase}) { $db_auto='yes'; }
+	else {  $db_auto='no'; }
+	print "Include InterBase support [$db_auto] ? ";	
+	$temp=read_in();			
+	if (($temp eq 'yes') || (($temp eq '') && ($db_auto eq 'yes'))) {
+		if ($db{ibase}) {
+			print "Enter InterBase base install directory [autodetect]: ";
+		} else {
+			print "Enter InterBase base install directory [/usr/interbase]: ";
+		}
 			
-			$temp=read_in();
-			if ($temp eq '') { $db_string="--with-oracle7"; } 
-			else { $db_string="--with-oracle7=$temp"; }
+		$temp=read_in_nolc();
+		if ($temp eq '') { $db_string .= " --with-ibase "; } 
+		else { $db_string .= " --with-ibase=$temp "; }
+	}			
+
+	if ($db{ora7}) { $db_auto='yes'; }
+	else {  $db_auto='no'; }
+	print "Include Oracle7 support [$db_auto] ? ";	
+	$temp=read_in();			
+	if (($temp eq 'yes') || (($temp eq '') && ($db_auto eq 'yes'))) {
+		print "Enter Oracle home directory [\$ORA_HOME]: ";
 			
-			last;
-		} elsif ($temp eq '8') {
-			if ($temp eq $db) {
-				print "Enter Oracle home directory [autodetect]: ";
-			} else {
-				print "Enter Oracle home directory [/oracle8/app/oracle/product/8.0.5]: ";
-			}
+		$temp=read_in_nolc();
+		if ($temp eq '') { $db_string .= " --with-oracle7 "; } 
+		else { $db_string .= " --with-oracle7=$temp "; }
+	}		
+
+	if ($db{ora8}) { $db_auto='yes'; }
+	else {  $db_auto='no'; }
+	print "Include Oracle8 support [$db_auto] ? ";	
+	$temp=read_in();			
+	if (($temp eq 'yes') || (($temp eq '') && ($db_auto eq 'yes'))) {
+		if ($db{ora8}) {
+			print "Enter Oracle home directory [autodetect]: ";
+		} else {
+			print "Enter Oracle home directory [/oracle8/app/oracle/product/8.0.5]: ";
+		}
 			
-			$temp=read_in();
-			if ($temp eq '') { $db_string="--with-oracle8"; } 
-			else { $db_string="--with-oracle8=$temp"; }
+		$temp=read_in_nolc();
+		if ($temp eq '') { $db_string .= " --with-oracle8 "; } 
+		else { $db_string .= " --with-oracle8=$temp "; }
+	}
 			
-			last;
-		} elsif ($temp eq '9') {
-			print "Enter Oracle home directory [\$ORA_HOME]: ";
+	if ($db{ora8i}) { $db_auto='yes'; }
+	else {  $db_auto='no'; }
+	print "Include Oracle8i support [$db_auto] ? ";	
+	$temp=read_in();			
+	if (($temp eq 'yes') || (($temp eq '') && ($db_auto eq 'yes'))) {
+		print "Enter Oracle home directory [\$ORA_HOME]: ";
 			
-			$temp=read_in();
-			if ($temp eq '') { $db_string="--with-oracle8i"; } 
-			else { $db_string="--with-oracle8i=$temp"; }
+		$temp=read_in_nolc();
+		if ($temp eq '') { $db_string .= " --with-oracle8i "; } 
+		else { $db_string .= " --with-oracle8i=$temp "; }
+	} 
+
+	if ($db{ctlib}) { $db_auto='yes'; }
+	else {  $db_auto='no'; }
+	print "Include Ct-Lib support [$db_auto] ? ";	
+	$temp=read_in();			
+	if (($temp eq 'yes') || (($temp eq '') && ($db_auto eq 'yes'))) {
+		if ($db{ctlib}) {
+			print "Enter Ct-Lib home directory [autodetect]: ";
+		} else {
+			print "Enter Ct-Lib home directory [/opt/sybase]: ";
+		}
 			
-			last;
-		} 
-	}	
+		$temp=read_in_nolc();
+		if ($temp eq '') { $db_string .= " --with-ctlib "; } 
+		else { $db_string .= " --with-ctlib=$temp "; }
+	}
+
+	if ($db{freetds}) { $db_auto='yes'; }
+	else {  $db_auto='no'; }
+	print "Include FreeTDS Ct-Lib support [$db_auto] ? ";	
+	$temp=read_in();			
+	if (($temp eq 'yes') || (($temp eq '') && ($db_auto eq 'yes'))) {
+		if ($db{freetds}) {
+			print "Enter FreeTDS Ct-Lib home directory [autodetect]: ";
+		} else {
+			print "Enter FreeTDS Ct-Lib home directory [/usr/local]: ";
+		}
+			
+		$temp=read_in_nolc();
+		if ($temp eq '') { $db_string .= " --with-freetds "; } 
+		else { $db_string .= " --with-freetds=$temp "; }
+	}
 }
 
 # ---------------------------
 # set_syslog
 # ---------------------------
 sub set_syslog {
+	print "Logging settings\n";
+	print "----------------\n";
+
 	while () {
 		print "Use syslog (yes) or stdout/stderr (no)? (yes/no) [$syslog]: ";
 		$temp=read_in();
@@ -521,7 +627,7 @@ sub set_syslog {
 	return if ($syslog eq 'no');
 	
 	print "Syslog facility (valid name from /usr/include/sys/syslog.h) [$syslog_facility]: ";
-	$temp=read_in();
+	$temp=read_in_nolc();
 	if (($temp eq '') || ($temp eq $syslog_facility)) {
 		return;
 	} else {
@@ -530,46 +636,19 @@ sub set_syslog {
 	}
 }
 
-# ---------------------------
-# set_threads
-# ---------------------------
-sub set_threads {
-	while () {
-		print "Enable Linux pthreads? (yes/no) [$linux_threads]: ";
-		$temp=read_in();
-		if (($temp eq '') || ($temp eq $linux_threads)) {
-			last;			
-		} elsif (($temp eq 'yes') || ($temp eq 'no')) {
-			$linux_threads=$temp;
-			last;
-		}
-	}
-	
-	return if ($linux_threads eq 'yes');
-
-	while () {
-		print "Enable FreeBSD pthreads? (yes/no) [$bsd_threads]: ";
-		$temp=read_in();
-		if (($temp eq '') || ($temp eq $bsd_threads)) {
-			last;			
-		} elsif (($temp eq 'yes') || ($temp eq 'no')) {
-			$bsd_threads=$temp;
-			last;
-		}
-	}
-}
-
 # ---------------------------
 # set_other
 # ---------------------------
 sub set_other {
+	print "Additional features\n";
+	print "-------------------\n";
 	while () {
-		print "Enable automatic Russian charset guesser? (yes/no) [$guesser]: ";
+		print "Enable Posix pthreads? (yes/no) [$pthreads]: ";
 		$temp=read_in();
-		if (($temp eq '') || ($temp eq $guesser)) {
+		if (($temp eq '') || ($temp eq $pthreads)) {
 			last;			
 		} elsif (($temp eq 'yes') || ($temp eq 'no')) {
-			$guesser=$temp;
+			$pthreads=$temp;
 			last;
 		}
 	}
@@ -597,27 +676,27 @@ sub set_other {
 	}
 
 	while () {
-		print "Enable AliasProg support? (yes/no) [$aliasprog]: ";
+		print "Enable HTTP Content-Encoding (zlib) support? (yes/no) [$zlib]: ";
 		$temp=read_in();
-		if (($temp eq '') || ($temp eq $aliasprog)) {
+		if (($temp eq '') || ($temp eq $zlib)) {
 			last;			
 		} elsif (($temp eq 'yes') || ($temp eq 'no')) {
-			$aliasprog=$temp;
-			last;
-		}
-	}
-
-	while () {
-		print "Enable extended NEWS features? (yes/no) [$ext_news]: ";
-		$temp=read_in();
-		if (($temp eq '') || ($temp eq $ext_news)) {
-			last;			
-		} elsif (($temp eq 'yes') || ($temp eq 'no')) {
-			$ext_news=$temp;
+			$zlib=$temp;
 			last;
 		}
 	}
 	
+	while () {
+		print "Enable DMALLOC support ? (yes/no) [$pthreads]: ";
+		$temp=read_in();
+		if (($temp eq '') || ($temp eq $dmalloc)) {
+			last;			
+		} elsif (($temp eq 'yes') || ($temp eq 'no')) {
+			$dmalloc=$temp;
+			last;
+		}
+	}
+
 	while () {
 		print "Enable OpenSSL support ? (yes/no) [$ssl]: ";
 		$temp=read_in();
@@ -631,87 +710,17 @@ sub set_other {
 	
 	if ($ssl eq 'yes') {
 		print "Enter SSL base install directory: [/usr/local/ssl]: ";
-		$temp=read_in();
+		$temp=read_in_nolc();
 		$ssl_dir=$temp;
 	}	
 }
 
-# ---------------------------
-# set_cache()
-# ---------------------------
-sub set_cache {	
-	while () {
-		print "Do you plan to use cache storage mode? (yes/no) [$cache]: ";
-		$temp=read_in();
-		if (($temp eq '') || ($temp eq $cache)) {
-			last;			
-		} elsif (($temp eq 'yes') || ($temp eq 'no')){
-			$cache=$temp;
-			last;
-		}
-	}
-	
-	return if ($cache eq 'no');
-	
-	while () {
-		print "Enable fast tag search in cache mode? (yes/no) [$fast_tag]: ";
-		$temp=read_in();
-		if (($temp eq '') || ($temp eq $fast_tag)) {
-			last;			
-		} elsif (($temp eq 'yes') || ($temp eq 'no')){
-			$fast_tag=$temp;
-			last;
-		}
-	}
-
-	while () {
-		print "Enable fast category search in cache mode? (yes/no) [$fast_cat]: ";
-		$temp=read_in();
-		if (($temp eq '') || ($temp eq $fast_cat)) {
-			last;			
-		} elsif (($temp eq 'yes') || ($temp eq 'no')){
-			$fast_cat=$temp;
-			last;
-		}
-	}
-
-	while () {
-		print "Enable fast site search in cache mode? (yes/no) [$fast_site]: ";
-		$temp=read_in();
-		if (($temp eq '') || ($temp eq $fast_site)) {
-			last;			
-		} elsif (($temp eq 'yes') || ($temp eq 'no')){
-			$fast_site=$temp;
-			last;
-		}
-	}
-
-	while () {
-		print "Enable phrase support in cache mode? (yes/no) [$phrase]: ";
-		$temp=read_in();
-		if (($temp eq '') || ($temp eq $phrase)) {
-			last;			
-		} elsif (($temp eq 'yes') || ($temp eq 'no')){
-			$phrase=$temp;
-			last;
-		}
-	}
-}
-
 # ---------------------------
 # set_shema()
 # ---------------------------
 sub set_shema {	
-	while () {
-		print "Enable exec: and cgi: virtual scheme support? (yes/no) [$cgi]: ";
-		$temp=read_in();
-		if (($temp eq '') || ($temp eq $cgi)) {
-			last;			
-		} elsif (($temp eq 'yes') || ($temp eq 'no')){
-			$cgi=$temp;
-			last;
-		}
-	}
+	print "URL parser settings\n";
+	print "-------------------\n";
 
 	while () {
 		print "Enable file:/ URL scheme support? (yes/no) [$file]: ";
@@ -775,6 +784,9 @@ sub set_shema {
 # set_libs
 # ---------------------------
 sub set_libs {
+	print "Compilation settings\n";
+	print "--------------------\n";
+
 	while () {
 		print "Build shared libraries? (yes/no) [$build_shared]: ";
 		$temp=read_in();
@@ -802,11 +814,20 @@ sub set_libs {
 # set_layout
 # ---------------------------
 sub set_layout {
-	print "Setting up directory layout:\n";
+	print "Layout settings\n";
+	print "---------------\n";
 
-	print "Please set installation path [$prefix]: ";
-	$temp=read_in();
-	$prefix=$temp if ($temp ne '');
+	while() {
+	    print "Please set installation path [$prefix]: ";
+	    $temp=read_in_nolc();
+	    last if ($temp eq '');
+	    if (abs_path($temp) eq abs_path('.')) {
+		print "Cannot install in current directory\n";
+	    } else {
+		$prefix=$temp if ($temp ne '');
+		last;
+	    }
+	}
 
 	update_dirs($prefix);
 	while() {
@@ -818,31 +839,31 @@ sub set_layout {
 			last;
 		} elsif ($temp eq 'yes') {
 			print "Please set User executables DIR [$bindir]: ";
-			$temp=read_in();
+			$temp=read_in_nolc();
 			$bindir=$temp if ($temp ne '');
 		
 			print "System executables DIR [$sbindir]: ";
-			$temp=read_in();
+			$temp=read_in_nolc();
 			$sbindir=$temp if ($temp ne '');
 			
 			print "Configuration data DIR [$sysconfdir]: ";
-			$temp=read_in();
+			$temp=read_in_nolc();
 			$sysconfdir=$temp if ($temp ne '');
 		
 			print "Modifiable data DIR: [$localstatedir]: ";
-			$temp=read_in();
+			$temp=read_in_nolc();
 			$localstatedir=$temp if ($temp ne '');
 		
 			print "Object code libraries DIR [$libdir]:";
-			$temp=read_in();
+			$temp=read_in_nolc();
 			$libdir=$temp if ($temp ne '');
 		
 			print "C header files DIR [$includedir]: ";
-			$temp=read_in();
+			$temp=read_in_nolc();
 			$includedir=$temp if ($temp ne '');
 		
 			print "Man documentation DIR [$mandir]: ";
-			$temp=read_in();
+			$temp=read_in_nolc();
 			$mandir=$temp if ($temp ne '');
 		} else {
 			next;
@@ -859,6 +880,15 @@ sub read_in {
 	return lc($temp);
 }
 
+# -------------------------------
+# read_in_nolc
+# -------------------------------
+sub read_in_nolc {
+	my $temp;
+	chomp($temp=<STDIN>);
+	return $temp;
+}
+
 # -------------------------------
 # update_dirs(prefix)
 # -------------------------------
