@@ -1,3 +1,20 @@
+/* Copyright (C) 2000-2002 Lavtech.com corp. All rights reserved.
+
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
+*/
+
 #include "udm_config.h"
 
 #ifdef USE_FTP
@@ -35,17 +52,22 @@
 #include "udm_xmalloc.h"   
 #include "udm_conf.h"   
 
-int UdmFTPGetReply(UDM_CONN *connp){
+/*#define DEBUG_FTP*/
+
+int Udm_ftp_get_reply(UDM_CONN *connp){
 	if (!connp->buf)
 		return -1;
 	
         return (atoi(connp->buf)/100);
 }
 
-int UdmFTPReadLine(UDM_CONN *connp){
+int Udm_ftp_read_line(UDM_CONN *connp){
 
-	if (socket_select(connp, NET_READ_TIMEOUT, 'r')){
-		UdmLog(connp->indexer, UDM_LOG_DEBUG, "ftp://%s (UdmFTPread_line-timeout-err): ", connp->hostname);
+	if (socket_select(connp, UDM_NET_READ_TIMEOUT, 'r')){
+#ifdef DEBUG_FTP
+	  fprintf(stderr, "ftp://%s (ftp_read_line-timeout-err): ", connp->hostname);
+		/*UdmLog(connp->indexer, UDM_LOG_DEBUG, "ftp://%s (ftp_read_line-timeout-err): ", connp->hostname);*/
+#endif
 		return -1;
 	}
 	
@@ -61,16 +83,16 @@ int UdmFTPReadLine(UDM_CONN *connp){
 	return 0;
 }
 
-int UdmFTPConnect(UDM_ENV * Conf,UDM_CONN *connp, char *hostname, int port, char *user, 
+int Udm_ftp_connect(UDM_ENV * Conf,UDM_CONN *connp, char *hostname, int port, char *user, 
 		char *passwd, int timeout){
 	size_t len;
 	
 	if (!connp)
 		return -1;
 
-	if (connp->connected == NET_CONNECTED)
-		UdmFTPClose(connp);	
-	connp->connected = NET_NOTCONNECTED;
+	if (connp->connected == UDM_NET_CONNECTED)
+		Udm_ftp_close(connp);	
+	connp->connected = UDM_NET_NOTCONNECTED;
 
 	if (!port)
 		connp->port = 21;
@@ -85,33 +107,33 @@ int UdmFTPConnect(UDM_ENV * Conf,UDM_CONN *connp, char *hostname, int port, char
 	connp->hostname = UdmXrealloc(connp->hostname, len+1);
 	udm_snprintf(connp->hostname, len+1, "%s", hostname);
 	
-	if (UdmFTPOpenControlPort(Conf,connp))
+	if (Udm_ftp_open_control_port(Conf,connp))
 		return -1;
-	if (UdmFTPLogin(connp, user, passwd))
+	if (Udm_ftp_login(connp, user, passwd))
 		return -1;
-	UdmFTPSetBinary(connp);
-	connp->connected = NET_CONNECTED;
+	Udm_ftp_set_binary(connp);
+	connp->connected = UDM_NET_CONNECTED;
 	return 0;
 }
 
-int UdmFTPOpenControlPort(UDM_ENV * Conf,UDM_CONN *connp){
+int Udm_ftp_open_control_port(UDM_ENV * Conf,UDM_CONN *connp){
 	int code;
 
-        if (UdmHostLookup(Conf,connp))
+        if (UdmHostLookup(&Conf->Hosts,connp)) /* FIXME: not reentrant */
 		return -1;
         if (socket_open(connp))
 		return -1;
 	if (socket_connect(connp))
 		return -1;
 	/* Read server response */
-	UdmFTPReadLine(connp);
-	code = UdmFTPGetReply(connp);
+	Udm_ftp_read_line(connp);
+	code = Udm_ftp_get_reply(connp);
 	if (code != 2)
 		return -1;
 	return 0;
 }
 
-int UdmFTPOpenDataPort( UDM_CONN *c, UDM_CONN *d){
+int Udm_ftp_open_data_port( UDM_CONN *c, UDM_CONN *d){
 	char buf[64];	
 	unsigned char *a, *p;
 	
@@ -136,15 +158,17 @@ int UdmFTPOpenDataPort( UDM_CONN *c, UDM_CONN *d){
 
 	udm_snprintf(buf, 64, "PORT %d,%d,%d,%d,%d,%d",
                 a[0], a[1], a[2], a[3], p[0], p[1]);
-	code = UdmFTPSendCmd(c, buf);
-	if (strncasecmp(c->buf, "200 PORT command successful.", 28)){
+	code = Udm_ftp_send_cmd(c, buf);
+	if (strncasecmp(c->buf, "200 ", 4)){
 		return -1;
 	}
+	d->user = c->user;
+	d->pass = c->pass;
 	return 0;
 }
 
 
-int UdmFTPSendCmd(UDM_CONN *connp, const char *cmd) {
+int Udm_ftp_send_cmd(UDM_CONN *connp, const char *cmd) {
         char *buf;
 	size_t len;
 
@@ -157,15 +181,21 @@ int UdmFTPSendCmd(UDM_CONN *connp, const char *cmd) {
 		UDM_FREE( buf);
 		return -1;
 	}
-	UdmLog(connp->indexer, UDM_LOG_DEBUG, "ftp://%s (cmd) : %s", connp->hostname, buf);
+#ifdef DEBUG_FTP
+	fprintf(stderr, "ftp://%s (cmd) : %s", connp->hostname, buf);
+	/*UdmLog(connp->indexer, UDM_LOG_DEBUG, "ftp://%s (cmd) : %s", connp->hostname, buf);*/
+#endif
 	UDM_FREE(buf);
-	if (UdmFTPReadLine(connp))
+	if (Udm_ftp_read_line(connp))
 		return -1;
-	UdmLog(connp->indexer, UDM_LOG_DEBUG, "ftp://%s (reply): %s", connp->hostname, connp->buf);
-    	return(UdmFTPGetReply(connp));
+#ifdef DEBUG_FTP
+	fprintf(stderr, "ftp://%s (reply): %s", connp->hostname, connp->buf);
+	/*UdmLog(connp->indexer, UDM_LOG_DEBUG, "ftp://%s (reply): %s", connp->hostname, connp->buf);*/
+#endif
+    	return(Udm_ftp_get_reply(connp));
 }
 
-static int UdmFTPExpectBytes(char *buf){
+static int ftp_expect_bytes(char *buf){
 	char *ch, *ch1;
 	int bytes;
 	
@@ -178,7 +208,7 @@ static int UdmFTPExpectBytes(char *buf){
 	return bytes;
 }
 
-int UdmFTPSendDataCmd(UDM_CONN *c, UDM_CONN *d, char *cmd, size_t max_doc_size){
+int Udm_ftp_send_data_cmd(UDM_CONN *c, UDM_CONN *d, char *cmd, size_t max_doc_size){
 	int code, bytes=0;
 	
 	if (!d)
@@ -188,12 +218,12 @@ int UdmFTPSendDataCmd(UDM_CONN *c, UDM_CONN *d, char *cmd, size_t max_doc_size){
 	d->hostname = c->hostname;
 	c->err = 0;
 	
-	if (UdmFTPOpenDataPort(c, d)){
+	if (Udm_ftp_open_data_port(c, d)){
 		socket_close(d);
 		return -1;
 	}
 
-	code = UdmFTPSendCmd(c, cmd);
+	code = Udm_ftp_send_cmd(c, cmd);
 	if (code == -1){
                 socket_close(d);
                 return -1;
@@ -202,39 +232,39 @@ int UdmFTPSendDataCmd(UDM_CONN *c, UDM_CONN *d, char *cmd, size_t max_doc_size){
 		socket_close(d);
 		return -1;
 	}
-	bytes = UdmFTPExpectBytes(c->buf);	
+	bytes = ftp_expect_bytes(c->buf);	
 	if (socket_accept(d)){
 		socket_close(d);
 		return -1;
 	}
 
 	if (socket_read(d, max_doc_size) < 0){
-		UdmLog(c->indexer, UDM_LOG_DEBUG, "ftp://%s (socket_read-err):", c->hostname);
+		/*UdmLog(c->indexer, UDM_LOG_DEBUG, "ftp://%s (socket_read-err):", c->hostname);*/
 		socket_close(d);
-		UdmFTPReadLine(c);
+		Udm_ftp_read_line(c);
 		return -1;
 	}
         socket_close(d);
  
-	/* Check if file too lage ABORT
+	/* Check if file too lage ABORT */
 	if (d->err == UDM_NET_FILE_TL){
-		if (UdmFTPAbort(c)){
+		if (Udm_ftp_abort(c)) {
 			socket_buf_clear(d);
 			return -1;
 		}
 	} 
-	*/	
+	
 
 	/* 226 Transfer complete. */
-	if (UdmFTPReadLine(c)){
+	if (Udm_ftp_read_line(c)){
 		/* Fixme: What to do if not receive "226 Transfer complete."? */
-		UdmLog(c->indexer, UDM_LOG_DEBUG, "ftp://%s (data-end-err): %d", d->hostname, d->buf_len);
-		UdmFTPClose(c);
+		/*UdmLog(c->indexer, UDM_LOG_DEBUG, "ftp://%s (data-end-err): %d", d->hostname, d->buf_len);*/
+		Udm_ftp_close(c);
 		if (bytes == d->buf_len )
 			return 0;
 		return -1;
 	}
-	code = UdmFTPGetReply(c);
+	code = Udm_ftp_get_reply(c);
 	if (code == -1){
                 return -1;
 	}else if ( code >3 ){
@@ -244,26 +274,32 @@ int UdmFTPSendDataCmd(UDM_CONN *c, UDM_CONN *d, char *cmd, size_t max_doc_size){
 	return 0;		
 }
 
-int UdmFTPLogin(UDM_CONN *connp, char *user, char *passwd){
+int Udm_ftp_login(UDM_CONN *connp, char *user, char *passwd){
 	char *buf;
 	char user_tmp[32], passwd_tmp[64];
 	int code;
 	size_t len;
 	
+	UDM_FREE(connp->user);
+	UDM_FREE(connp->pass);
 	if (!user)
 		udm_snprintf(user_tmp, 32, "anonymous");
-	else
+	else {
 		udm_snprintf(user_tmp, 32, "%s", user);
+		connp->user = (char*)strdup(user);
+	}
 	    
 	if (!passwd)
-		udm_snprintf(passwd_tmp, 64, "%s-%s@search.mnogo.ru", PACKAGE, VERSION);
-	else
+		udm_snprintf(passwd_tmp, 64, "%s-%s@mnogosearch.org", PACKAGE, VERSION);
+	else {
 		udm_snprintf(passwd_tmp, 32, "%s", passwd);
+		connp->pass = (char*)strdup(passwd);
+	}
 	
 	len = strlen(user_tmp)+strlen("USER ");
 	buf = (char *) UdmXmalloc(len+1);
 	udm_snprintf(buf, len+1, "USER %s", user_tmp);
-	code = UdmFTPSendCmd(connp, buf);
+	code = Udm_ftp_send_cmd(connp, buf);
 	UDM_FREE(buf);
 	if (code == -1)
 		return -1;
@@ -273,14 +309,14 @@ int UdmFTPLogin(UDM_CONN *connp, char *user, char *passwd){
 	len = strlen(passwd_tmp)+strlen("PASS ");
 	buf = (char *) UdmXmalloc(len+1);
 	udm_snprintf(buf, len+1, "PASS %s", passwd_tmp);
-	code = UdmFTPSendCmd( connp, buf);
+	code = Udm_ftp_send_cmd( connp, buf);
 	UDM_FREE(buf);
 	if (code > 3)
 		return -1;
 	return 0;
 }
 
-static int UdmFTPParseList(UDM_CONN *connp, char *path){
+static int ftp_parse_list(UDM_CONN *connp, char *path){
 	char *line, *buf_in, *ch, *buf_out, *tok, *fname;
         int len_h, len_f,len_p, i;
 	char *dir;
@@ -290,12 +326,13 @@ static int UdmFTPParseList(UDM_CONN *connp, char *path){
 		return 0;
         buf_in = connp->buf;
 	/* 22 = strlen(<a href=\"ftp://%s%s%s/\"></a>)*/
-        len_h = strlen(connp->hostname)+22;
+        len_h = strlen(connp->hostname) + ((connp->user) ? strlen(connp->user) : 0) + ((connp->pass) ? strlen(connp->pass) : 0) + 2 + 22;
         len_p = strlen(path);
         cur_len = 0;
         buf_len = connp->buf_len;
         buf_out = UdmXmalloc(buf_len);
-	line=UdmGetToken(buf_in,"\r\n",&tok);
+	buf_out[0] = '\0';
+	line = udm_strtok_r(buf_in,"\r\n",&tok);
         do{
     		if (!(fname = strtok(line, " ")))
 			continue;
@@ -308,7 +345,7 @@ static int UdmFTPParseList(UDM_CONN *connp, char *path){
 		len = 0 ;
 		len_f = len_h + len_p + strlen(fname);
 	        if ((cur_len+len_f) >= buf_len){
-			buf_len += NET_BUF_SIZE;
+			buf_len += UDM_NET_BUF_SIZE;
 			buf_out = UdmXrealloc(buf_out, buf_len);
 		}
 					
@@ -317,7 +354,9 @@ static int UdmFTPParseList(UDM_CONN *connp, char *path){
 				if (!fname || !strcmp(fname, ".") || !strcmp(fname, ".."))
 				        break;
 				len = len_f;
-    				udm_snprintf(buf_out+cur_len, len+1, "<a href=\"ftp://%s%s%s/\"></a>",
+    				udm_snprintf(UDM_STREND(buf_out) /*buf_out+cur_len*/, len+1, "<a href=\"ftp://%s%s%s%s%s%s%s/\"></a>\n",
+					 (connp->user) ? connp->user : "", (connp->user) ? ":" : "",
+					 (connp->pass) ? connp->pass : "", (connp->user || connp->pass) ? "@" : "",
 					    connp->hostname, path, fname);
 				break;
     	    	        case 'l':
@@ -328,9 +367,11 @@ static int UdmFTPParseList(UDM_CONN *connp, char *path){
 				dir = malloc(len+1);
 				udm_snprintf(dir, len+1, "%s", fname);
 				len = len_h + len_p + strlen(dir);
-				udm_snprintf(buf_out+cur_len, len+1, "<a href=\"ftp://%s%s%s/\"></a>", 
+				udm_snprintf(UDM_STREND(buf_out)/*buf_out+cur_len*/, len+1, "<a href=\"ftp://%s%s%s%s%s%s%s/\"></a>\n", 
+					 (connp->user) ? connp->user : "", (connp->user) ? ":" : "",
+					 (connp->pass) ? connp->pass : "", (connp->user || connp->pass) ? "@" : "",
 					    connp->hostname, path, dir);
-				free(dir);
+				UDM_FREE(dir);
 				/*ch +=4;*/
 				/* Check if it is absolute link */
 /*				if ((ch[0] == '/') || (ch[0] == '\\') ||
@@ -347,14 +388,16 @@ static int UdmFTPParseList(UDM_CONN *connp, char *path){
 				break;
     	    	        case '-':
 				len =  len_f; 
-		    	        udm_snprintf(buf_out+cur_len, len+1, "<a  href=\"ftp://%s%s%s\"></a>", 
+		    	        udm_snprintf(UDM_STREND(buf_out)/*buf_out+cur_len*/, len+1, "<a  href=\"ftp://%s%s%s%s%s%s%s\"></a>\n", 
+					 (connp->user) ? connp->user : "", (connp->user) ? ":" : "",
+					 (connp->pass) ? connp->pass : "", (connp->user || connp->pass) ? "@" : "",
 					    connp->hostname, path, fname);
 
 				break;
 		}
 		cur_len += len;
 		
-	}while( (line=UdmGetToken(NULL, "\r\n", &tok)));
+	}while( (line = udm_strtok_r(NULL, "\r\n", &tok)));
 
 	if (cur_len+1 > connp->buf_len_total){
 		connp->buf_len_total = cur_len;  
@@ -366,7 +409,7 @@ static int UdmFTPParseList(UDM_CONN *connp, char *path){
 	return 0;
 }
 
-int UdmFTPList(UDM_CONN *c, UDM_CONN *d, char *path, char *filename, size_t max_doc_size){
+int Udm_ftp_list(UDM_CONN *c, UDM_CONN *d, char *path, char *filename, size_t max_doc_size){
 	char *cmd;
 	size_t len;
 	
@@ -379,17 +422,17 @@ int UdmFTPList(UDM_CONN *c, UDM_CONN *d, char *path, char *filename, size_t max_
     		udm_snprintf(cmd, len+1, "LIST %s", filename);
 	}
 
-	if (UdmFTPSendDataCmd(c, d, cmd,max_doc_size)== -1){
+	if (Udm_ftp_send_data_cmd(c, d, cmd,max_doc_size)== -1){
 		UDM_FREE(cmd);
-		UdmLog(c->indexer, UDM_LOG_DEBUG, "(UdmFTPList-err)->%s", d->buf);
+		/*UdmLog(c->indexer, UDM_LOG_DEBUG, "(ftp_list-err)->%s", d->buf);*/
 		return -1;
 	}
 	UDM_FREE(cmd);
-	UdmFTPParseList(d, path);
+	ftp_parse_list(d, path);
 	return 0;
 }
 
-int UdmFTPGet(UDM_CONN *c, UDM_CONN *d, char *path, size_t max_doc_size){
+int Udm_ftp_get(UDM_CONN *c, UDM_CONN *d, char *path, size_t max_doc_size){
 	char *cmd;
 	size_t len;
 	
@@ -400,7 +443,7 @@ int UdmFTPGet(UDM_CONN *c, UDM_CONN *d, char *path, size_t max_doc_size){
 	cmd = UdmXmalloc(len+1);
         udm_snprintf(cmd, len+1, "RETR %s", path);
 
-	if (UdmFTPSendDataCmd(c, d, cmd,max_doc_size) == -1){
+	if (Udm_ftp_send_data_cmd(c, d, cmd, max_doc_size) == -1 && d->err != UDM_NET_FILE_TL) {
 		UDM_FREE(cmd);
 		return -1;
 	}
@@ -408,7 +451,7 @@ int UdmFTPGet(UDM_CONN *c, UDM_CONN *d, char *path, size_t max_doc_size){
 	return 0;
 }
 
-int UdmFTPMdtm(UDM_CONN *c, char *path){
+int Udm_ftp_mdtm(UDM_CONN *c, char *path){
 	char *cmd;
 	int code;
 	size_t len; 
@@ -420,7 +463,7 @@ int UdmFTPMdtm(UDM_CONN *c, char *path){
 	cmd = UdmXmalloc(len+1);
 	udm_snprintf(cmd, len+1, "MDTM %s", path);
 
-	code = UdmFTPSendCmd(c, cmd);
+	code = Udm_ftp_send_cmd(c, cmd);
 	UDM_FREE(cmd);
 	if (code == -1){
                 return -1;
@@ -431,14 +474,54 @@ int UdmFTPMdtm(UDM_CONN *c, char *path){
 	return (UdmFTPDate2Time_t(c->buf));
 }
 
-int UdmFTPSetBinary(UDM_CONN *c){
+size_t Udm_ftp_size(UDM_CONN *c, char *path) {
+	char *cmd;
+	int code;
+	size_t len; 
+	
+	if (!path)
+		return -1;
+		
+    	len = strlen(path) + 5;
+	cmd = UdmXmalloc(len + 1);
+	udm_snprintf(cmd, len + 1, "SIZE %s", path);
+
+	code = Udm_ftp_send_cmd(c, cmd);
+	UDM_FREE(cmd);
+	if (code == -1){
+                return -1;
+	}else if ( code >3 ){
+		c->err = code;
+		return -1;
+	}
+	sscanf(c->buf, "213 %u", &len);
+	return len;
+}
+
+int Udm_ftp_rest(UDM_CONN *c, size_t rest) {
+	char cmd[64];
+	int code;
+	
+	udm_snprintf(cmd, 63, "REST %u", rest);
+
+	code = Udm_ftp_send_cmd(c, cmd);
+	if (code == -1){
+                return -1;
+	}else if ( code >3 ){
+		c->err = code;
+		return -1;
+	}
+	return 0;
+}
+
+int Udm_ftp_set_binary(UDM_CONN *c){
 	char *cmd;
 	int code;
 	
 	cmd = UdmXmalloc(7);
 	sprintf(cmd, "TYPE I");
 
-	code = UdmFTPSendCmd(c, cmd);
+	code = Udm_ftp_send_cmd(c, cmd);
 	UDM_FREE(cmd);
 	if (code == -1){
                 return -1;
@@ -449,7 +532,7 @@ int UdmFTPSetBinary(UDM_CONN *c){
 	return 0;
 }
 
-int UdmFTPCwd(UDM_CONN *c, char *path){
+int Udm_ftp_cwd(UDM_CONN *c, char *path){
 	char *cmd;
 	int code;
 	size_t len;
@@ -461,7 +544,7 @@ int UdmFTPCwd(UDM_CONN *c, char *path){
 	cmd = UdmXmalloc(len+1);
 	udm_snprintf(cmd, len+1, "CWD %s", path);
 
-	code = UdmFTPSendCmd(c, cmd);
+	code = Udm_ftp_send_cmd(c, cmd);
 	UDM_FREE(cmd);
 	if (code == -1){
                 return -1;
@@ -472,20 +555,20 @@ int UdmFTPCwd(UDM_CONN *c, char *path){
 	return 0;
 }
 
-int UdmFTPClose(UDM_CONN *connp){
+int Udm_ftp_close(UDM_CONN *connp){
 	int code;
 
-	if (connp->connected == NET_CONNECTED){
-		code = UdmFTPSendCmd(connp, "QUIT");
+	if (connp->connected == UDM_NET_CONNECTED){
+		code = Udm_ftp_send_cmd(connp, "QUIT");
 	}
-	connp->connected = NET_NOTCONNECTED;
+	connp->connected = UDM_NET_NOTCONNECTED;
 	socket_close(connp);
 	if (connp->connp)
 		socket_close(connp->connp);
 	return 0;
 }
 
-int UdmFTPAbort(UDM_CONN *connp){
+int Udm_ftp_abort(UDM_CONN *connp){
 	int code;
 
 	socket_buf_clear(connp->connp);
@@ -495,7 +578,7 @@ int UdmFTPAbort(UDM_CONN *connp){
 	if (socket_write(connp, "\xF2"))
 		return -1;
 	
-	code = UdmFTPSendCmd(connp, "ABOR");
+	code = Udm_ftp_send_cmd(connp, "ABOR");
 	socket_buf_clear(connp->connp);
 
 	if (code !=4)
