@@ -1,3 +1,20 @@
+/* Copyright (C) 2000-2002 Lavtech.com corp. All rights reserved.
+
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
+*/
+
 #include "udm_config.h"
 
 #include <stdio.h>
@@ -21,36 +38,34 @@
 #include <fcntl.h>
 #include <sys/stat.h>
 #include <errno.h>
+#include <signal.h>
 
 #include "udm_common.h"
 #include "udm_utils.h"
 #include "udm_parser.h"
 #include "udm_xmalloc.h"
 #include "udm_log.h"
+#include "udm_vars.h"
+#include "udm_wild.h"
 
-
-__INDLIB__ int UdmAddParser(UDM_ENV * Conf,char *from_mime,char *to_mime,char *cmd){
-	if(Conf->nparsers){
-		Conf->parsers=(UDM_PARSER*)(realloc(Conf->parsers,(Conf->nparsers+1)*sizeof(UDM_PARSER)));
-	}else{
-		Conf->parsers=(UDM_PARSER*)(malloc(sizeof(UDM_PARSER)));
-	}
-	Conf->parsers[Conf->nparsers].from_mime=strdup(from_mime);
-	Conf->parsers[Conf->nparsers].to_mime=strdup(to_mime);
-	Conf->parsers[Conf->nparsers].cmd=strdup(cmd);
-	Conf->nparsers++;
+__C_LINK int __UDMCALL UdmParserAdd(UDM_PARSERLIST *List,UDM_PARSER *P){
+	List->Parser=(UDM_PARSER*)(realloc(List->Parser,(List->nparsers+1)*sizeof(UDM_PARSER)));
+	List->Parser[List->nparsers].from_mime = (char*)strdup(P->from_mime);
+	List->Parser[List->nparsers].to_mime = (char*)strdup(P->to_mime);
+	List->Parser[List->nparsers].cmd = (char*)strdup(P->cmd);
+	List->nparsers++;
 	return 0;
 }
 
-void UdmFreeParsers(UDM_ENV * Env){
+__C_LINK void __UDMCALL UdmParserListFree(UDM_PARSERLIST *List){
 	size_t i;
-	for(i=0;i<Env->nparsers;i++){
-		UDM_FREE(Env->parsers[i].from_mime);
-		UDM_FREE(Env->parsers[i].to_mime);
-		UDM_FREE(Env->parsers[i].cmd);
+	for(i=0;i<List->nparsers;i++){
+		UDM_FREE(List->Parser[i].from_mime);
+		UDM_FREE(List->Parser[i].to_mime);
+		UDM_FREE(List->Parser[i].cmd);
 	}
-	UDM_FREE(Env->parsers)
-	Env->nparsers=0;
+	UDM_FREE(List->Parser)
+	List->nparsers=0;
 }
 
 
@@ -58,13 +73,30 @@ void UdmFreeParsers(UDM_ENV * Env){
 
 /* Parser1: from STDIN to STDOUT */
 
-#if (WIN32|WINNT)
-static char *parse1(UDM_AGENT * Agent,char *buf,size_t buflen,char *cmd,size_t maxlen,char *url){
+#ifdef WIN32
+static char *parse1(UDM_AGENT * Agent, char *buf, size_t buflen, char *cmd, size_t maxlen, const char *url){
 	UdmLog(Agent,UDM_LOG_ERROR,"This type of parser isn't supported");
 	return NULL;
 }
 #else
-static char *parse1(UDM_AGENT * Agent,char *buf,size_t buflen,char *cmd,size_t maxlen,char *url){
+
+static void sighandler(int sign);
+static void init_signals(void){
+	/* Set up signals handler*/
+	UdmSignal(SIGALRM, sighandler);
+}
+static void sighandler(int sign){
+	switch(sign){
+	        case SIGALRM:
+		        _exit(0);
+			break;
+		default:
+			break;
+	}
+	init_signals();
+}
+
+static char *parse1(UDM_AGENT * Agent, char *buf, size_t buflen, char *cmd, size_t maxlen, const char *url){
 	char *result = NULL;
 	int wr[2];
 	int rd[2];    
@@ -88,7 +120,7 @@ static char *parse1(UDM_AGENT * Agent,char *buf,size_t buflen,char *cmd,size_t m
 
 	if (pid>0){
 		/* Parent process */
-	        char string[UDMSTRSIZ];
+	        char string[1024];
 
 		/* Close other pipe ends */
 		close(wr[0]);
@@ -136,6 +168,9 @@ static char *parse1(UDM_AGENT * Agent,char *buf,size_t buflen,char *cmd,size_t m
 			/* Connect pipe to stdin */
 			dup2(wr[0],STDIN_FILENO);
 
+			alarm((unsigned int) UdmVarListFindInt(&Agent->Conf->Vars, "ParserTimeOut", 300) );
+			init_signals();
+
 			system(cmd);
 			exit(0);
 		}
@@ -145,21 +180,21 @@ static char *parse1(UDM_AGENT * Agent,char *buf,size_t buflen,char *cmd,size_t m
 #endif
 
 /* Parser2: from FILE to STDOUT */
-static char *parse2(UDM_AGENT * Agent,char *buf,char *cmd,size_t maxlen,char *url){
+static char *parse2(UDM_AGENT * Agent, char *buf, char *cmd, size_t maxlen, const char *url){
 	FILE *f;
 	char *result = NULL;
 
 	result=buf;
 	memset(result,0,maxlen);
 
-#if (WIN32|WINNT)
+#ifdef WIN32
 	f=_popen(cmd,"rb");
 #else
 	f=popen(cmd,"r");
 #endif
 	if(f){
 		int fd;
-	        char string[UDMSTRSIZ];
+	        char string[1024];
 		
 		fd=fileno(f);
 		memset(string,0,sizeof(string));
@@ -167,7 +202,7 @@ static char *parse2(UDM_AGENT * Agent,char *buf,char *cmd,size_t maxlen,char *ur
 			strncat(result,string,maxlen-strlen(result));
 			memset(string,0,sizeof(string));
 		}
-#if (WIN32|WINNT)
+#ifdef WIN32
 		_pclose(f);
 #else
 		pclose(f);
@@ -180,7 +215,7 @@ static char *parse2(UDM_AGENT * Agent,char *buf,char *cmd,size_t maxlen,char *ur
 }
 
 /* Parser3: from FILE to FILE */
-static char *parse3(UDM_AGENT * Agent,char *buf,char *cmd,size_t maxlen,char *url,char *to_file){
+static char *parse3(UDM_AGENT * Agent, char *buf, char *cmd, size_t maxlen, const char *url, char *to_file){
 	char *result = NULL;
 	int fd;
 
@@ -200,11 +235,11 @@ static char *parse3(UDM_AGENT * Agent,char *buf,char *cmd,size_t maxlen,char *ur
 }
 
 /* Parser4: from STDIN to FILE */
-static char *parse4(UDM_AGENT * Agent,char *buf,size_t buflen,char *cmd,size_t maxlen,char *url,char *to_file){
+static char *parse4(UDM_AGENT * Agent, char *buf, size_t buflen, char *cmd, size_t maxlen, const char *url, char *to_file){
 	FILE *f;
 	char *result = NULL;
 
-#if (WIN32|WINNT)
+#ifdef WIN32
 	f=_popen(cmd,"wb");
 #else
 	f=popen(cmd,"w");
@@ -215,7 +250,7 @@ static char *parse4(UDM_AGENT * Agent,char *buf,size_t buflen,char *cmd,size_t m
 		
 		fd=fileno(f);
 		write(fd, buf, buflen);
-#if (WIN32|WINNT)
+#ifdef WIN32
 		_pclose(f);
 #else
 		pclose(f);
@@ -237,12 +272,12 @@ static char *parse4(UDM_AGENT * Agent,char *buf,size_t buflen,char *cmd,size_t m
 	return result;
 }
 
-static char *parse_file (UDM_AGENT * Agent,UDM_PARSER * parser,char *buf, size_t length, size_t maxlen,char * url){
-	char cmd[UDMSTRSIZ]="";
-	char *result;
+static char *parse_file (UDM_AGENT * Agent, UDM_PARSER * parser, char *buf, size_t length, size_t maxlen, const char * url){
+	char cmd[1024]="";
+	char *result=NULL;
 	char *arg1pos,*arg2pos;
 	int parser_type;
-	char fn0[1024]="";
+	char fn0[1024]="/tmp/" "indXXXXXX";
 	char fn1[1024]="";
 	char * fnames[2];
 
@@ -250,12 +285,12 @@ static char *parse_file (UDM_AGENT * Agent,UDM_PARSER * parser,char *buf, size_t
 	arg2pos=strstr(parser->cmd,"$2");
 
 	/* Build temp file names and command line */
-#if (WIN32||WINNT)
+#ifdef WIN32
 	/* Windows generates something strange in tmpnam() */
 	/* So create the name ourself                      */
 	sprintf(fn0, ".\\parser_tmp%d",Agent->handle);
 #else
-	tmpnam(fn0);
+	sprintf(fn0,"/tmp/ind.%d.%d", Agent->handle, getpid());
 #endif
 	strcpy(fn1,fn0);
 	fnames[0]=strcat(fn0,".in");
@@ -266,8 +301,11 @@ static char *parse_file (UDM_AGENT * Agent,UDM_PARSER * parser,char *buf, size_t
 		int fd;
 
 		/* Create temporary file */
+#ifdef WIN32
+		umask((int)022);
+#else
 		umask((mode_t)022);
-
+#endif
 		fd=open(fnames[0],O_RDWR|O_CREAT|UDM_BINARY,UDM_IWRITE);
 		/* Write to the temporary file */
 		write(fd,buf,length);
@@ -300,16 +338,19 @@ static char *parse_file (UDM_AGENT * Agent,UDM_PARSER * parser,char *buf, size_t
 	return result;
 }
 
-char *UdmExecParser(UDM_AGENT * Agent,char *from_mime, int *to_mime, char *buf, size_t length, size_t maxlen,char * url){
+__C_LINK UDM_PARSER * __UDMCALL UdmParserFind(UDM_PARSERLIST *List,const char *mime_type){
 	size_t i;
-
-	for(i=0;i<Agent->Conf->nparsers;i++){
-		if(!UdmStrCaseMatch(from_mime,Agent->Conf->parsers[i].from_mime)){
-			*to_mime=i;
- 			return parse_file(Agent,&(Agent->Conf->parsers[i]),buf,length,maxlen,url);
-		}
-    	}
+	for(i=0;i<List->nparsers;i++){
+		if(!UdmWildCaseCmp(mime_type,List->Parser[i].from_mime))
+			return &List->Parser[i];
+	}
     	return NULL;
 }
 
+char *UdmParserExec(UDM_AGENT * Agent,UDM_PARSER *P,UDM_DOCUMENT *Doc){
+	size_t maxlen=(size_t)(Doc->Buf.maxsize-(Doc->Buf.content-Doc->Buf.buf));
+	size_t srclen=(size_t)(Doc->Buf.size-(Doc->Buf.content-Doc->Buf.buf));
+	return parse_file(Agent,P,Doc->Buf.content,srclen,maxlen,UdmVarListFindStr(&Doc->Sections,"URL",""));
+}
+
 #endif
